pub struct BTreePageCache { /* private fields */ }Expand description
Shared page cache for B+Tree index pages
This type allows the cache to be shared across multiple BTreeManager instances, enabling cache persistence across backend reopen cycles.
Implementations
Sourceimpl BTreePageCache
impl BTreePageCache
Sourcepub fn with_default_capacity() -> Self
pub fn with_default_capacity() -> Self
Create a new shared page cache with default capacity (64 pages)
Sourcepub fn get(&self, page_id: u64) -> Option<IndexPage>
pub fn get(&self, page_id: u64) -> Option<IndexPage>
Get a page from the cache (returns clone if found)