pub type PooledConnection = PooledConnection<ConnectionManager>;Expand description
Type alias for a pooled connection that can be checked out from the pool.
Aliased Type
pub struct PooledConnection { /* private fields */ }pub type PooledConnection = PooledConnection<ConnectionManager>;Type alias for a pooled connection that can be checked out from the pool.
pub struct PooledConnection { /* private fields */ }